"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.metaExternals = void 0;
const Imports_1 = require("../Compiler/Passes/Imports");
const Lazify_1 = require("../Compiler/Passes/Lazify");
const Utils_1 = require("../Compiler/Utils");
const Normalizer_1 = require("../Normalizer/Normalizer");
const JSTranslator_1 = require("../Translator/JSTranslator");
const trace = (log) => (query, normalizer, externals) => {
    return Normalizer_1.traceNormalize(Lazify_1.Inst(query.args[0]), normalizer, externals, (t) => {
        log(Utils_1.showTerm(t));
    });
};
const equ = (s, t) => {
    return Utils_1.termsEq(s, t) ? Imports_1.True : Imports_1.False;
};
const nativeMetaExternals = (log) => {
    return {
        trace: trace(log),
        equ: t => { const [a, b] = t.args; return equ(a, b); },
    };
};
const traceNotAvailableMsg = '@trace is not available with this normalizer';
const jsMetaExternals = {
    trace: (name) => `function ${name}() { throw new Error("${traceNotAvailableMsg}"); }`,
    equ: (name) => `
        function ${name}(a, b) {
            if (isFun(a) && isFun(b) && a.name === b.name && a.args.length === b.args.length) {
              for (let i = 0; i < a.args.length; i++) {
                  if (${name}(a.args[i], b.args[i]).name === "False") {
                      return ${JSTranslator_1.nullaryVarName('False')};
                  }
              }
              return ${JSTranslator_1.nullaryVarName('True')};
            }
          
            return a === b ? ${JSTranslator_1.nullaryVarName('True')} : ${JSTranslator_1.nullaryVarName('False')};
        }`,
};
const haskellMetaExternals = {
    trace: (name) => `${name} = error "${traceNotAvailableMsg}"`,
    equ: (name) => `${name} a b = if a == b then (Fun "True" []) else (Fun "False" [])`
};
const ocamlMetaExternals = {
    trace: (name) => `let ${name} () = failwith "${traceNotAvailableMsg}";;`,
    equ: (name) => `let ${name} (a, b) = if a = b then Fun ("True", []) else Fun ("False", []);;`
};
const log = (msg) => { console.log(msg); };
exports.metaExternals = (nativeLog = log) => target => {
    return {
        'native': nativeMetaExternals(nativeLog),
        'js': jsMetaExternals,
        'ocaml': ocamlMetaExternals,
        'haskell': haskellMetaExternals
    }[target];
};
