"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HaskellTranslator = void 0;
const Utils_1 = require("../Compiler/Utils");
const Translator_1 = require("./Translator");
class HaskellTranslator extends Translator_1.Translator {
    constructor(trs, externals) {
        super(trs, externals, HaskellTranslator.reservedKeywords);
        this.header = [
            "import Data.List (intercalate, map)",
            "data Term = Var String | Fun String [Term] deriving (Eq)",
            `instance Show Term where
            show (Var x) = x
            show (Fun f []) = f
            show (Fun f ts) = f ++ "(" ++ (intercalate ", " (map show ts)) ++ ")"
        `,
        ];
    }
    translateTerm(term) {
        if (Utils_1.isVar(term))
            return this.renameVar(term);
        return `(Fun "${term.name}" [${term.args.map(t => this.translateTerm(t)).join(", ")}])`;
    }
    callTerm(term) {
        if (Utils_1.isVar(term))
            return term;
        if (!this.isDefined(term.name)) {
            return this.translateTerm(Utils_1.fun(term.name, ...term.args.map(t => this.callTerm(t))));
        }
        const args = `${term.args.map(t => this.callTerm(t)).join(' ')}`;
        return `(${term.name} ${args})`;
    }
    translateRules(name, rules) {
        const res = [];
        for (const [lhs, rhs] of rules) {
            const args = `${lhs.args.map(t => this.translateTerm(t)).join(' ')}`;
            res.push(`${name} ${args} = ${this.callTerm(rhs)}`);
        }
        if (!Utils_1.hasMostGeneralRule(rules)) {
            const args = Utils_1.genVars(Utils_1.arity(rules));
            res.push(`${name} ${args.join(' ')} = ${this.translateTerm(Utils_1.fun(name, ...args))}`);
        }
        return res.join('\n');
    }
}
exports.HaskellTranslator = HaskellTranslator;
HaskellTranslator.reservedKeywords = new Set([
    'case', 'class', 'data', 'deriving', 'do', 'else', 'if',
    'import', 'in', 'infix', 'infixl', 'infixr', 'instance',
    'let', 'of', 'module', 'newtype', 'then', 'type', 'where'
]);
